
#include <avr/io.h>
#include "uart.h"
#include <avr/interrupt.h>
#include <util/delay.h>

FILE uart_output = FDEV_SETUP_STREAM(uart_putc, NULL, _FDEV_SETUP_WRITE);
FILE uart_input  = FDEV_SETUP_STREAM(NULL, uart_getc, _FDEV_SETUP_READ); 

static volatile int vstup_minuty = 0;
static volatile int vstup_sekundy = 0;
static volatile int cas = 0;
static volatile int vystup_min = 0;
static volatile int vystup_sek = 0;
static volatile int stop = 0;


#define SW1 PD7		// tlacitko

#define set_bit(ADDRESS,BIT) (ADDRESS |= (1<<BIT))
#define clear_bit(ADDRESS,BIT) (ADDRESS &= ~(1<<BIT))


enum states { Off, Down, On, Up };		// stavy tlacitka

ISR (TIMER1_OVF_vect)          
 {
   TCNT1 = 0xC2F7;		// nastavenie pocitadla  
   
   if (cas>=0)
   {
	   cas--;
   }
 }
 
void tone(int note, int delay)
{
	OCR0A = note;
	
	for (int i=1; i<=delay; i++)
     _delay_ms(1);
}

 
void hudba()		// funkcia prehravania melodie
{
	int c = 78;
    int h = 104;
    int p = 0;
		  
    tone(c,250);
	tone(p,50);
	tone(h,250);
	tone(p,250);	  
  
}

                              
int main(void)
{
  uart_init();
  stdout = &uart_output;           // teraz funguje printf()
  stdin = &uart_input;            // teraz funguje scanf()
  
  set_bit(PORTD,SW1);       // zapnutie pull-up rezistora
  clear_bit(DDRD,SW1);     //  nastavenie pinu SW1 ako vstup
  
  enum states Tlacitko = Off;
  
  
  //************************** T1 ***********************//
  
  TCNT1 = 0xC2F7;    
  TCCR1B = 0b00000101;		// externy zdroj hodin na pin T1 reagujuci na nabeznu hranu
  
  
  // konfiguracia prerusovacieho systemu
   PCMSK2 |= (1<<PCINT21);
   PCICR |= (1<<PCIE2);		
   sei();					// povolenie preruseni
	
  // pretecenie
   TIMSK1 = (1<<TOIE1);    // povolenie pretecenia Timer 1
   sei();                  // povolenie preruseni
  
  
 //********************* hudba *************************//
 
   DDRD   |= (1 << PD6);                      // port D.6 pin ako vystup
   TCCR0A =  (1 << COM0A0) | (1 << WGM01);    // Timer 0 v rezime CTC
   TCCR0B = (1 << CS02);                      // nastavenie preddelicky na 256
   
   
   
  while(1)
  {	
	 puts("Zadaj cas (min:sek)\r");
     scanf("%d:%d",&vstup_minuty,&vstup_sekundy); // nacitanie casu
     cas = vstup_minuty * 60 + vstup_sekundy;     // prepocitanie na sekundy
	 
	 while(cas>=0)
	 {
		 vystup_min = cas / 60;
		 vystup_sek = cas % 60;
		 printf("%d:%d\r", vystup_min, vystup_sek);
		 delay(1000);
		 if(cas !=-1)
		 {
			 printf("     \r");
		 }
		 
	 }
	 stop = 0;
     while(stop == 0)
	 {
		 hudba();
		if ( (Tlacitko == Off) &&  bit_is_clear(PIND,SW1)  )
	     {
			 
		    Tlacitko = Down;
				           	  
	     }
	     else if ( (Tlacitko == Down) &&  bit_is_clear(PIND,SW1)  )
	     {
			 
  		     _delay_ms(10);                 
		    if ( bit_is_clear(PIND,SW1) )  	
		     Tlacitko = On;
			 
	     }
	     else if  ( (Tlacitko == On) &&  bit_is_set(PIND,SW1) ) 	  
	     {
	   	  
	   	    Tlacitko = Up;
			
	     }
	     else if (  (Tlacitko == Up) &&  bit_is_set(PIND,SW1) )
	     {
			 
		     _delay_ms(10); 
		    if ( bit_is_set(PIND,SW1) )	
		    Tlacitko = Off;
			
	     }
		 
		 if(Tlacitko == Down)
		 {
			 
			stop = 1;
			Tlacitko = Off;
			
		 }
	 }
  }
  return(0);
}
